/*-
 * Copyright (C) @BABOLO  2002 Dec 04
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2002 Dec 04\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: pgoblin.c,v 1.2 2002/12/16 23:42:47 babolo Exp $";
#endif /* not lint */
#include <sys/types.h>
#include <sysexits.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <err.h>
#include <libpq-fe.h>
#include "pgoblin.h"

int v_flag;

void
usage(int ex) {
    fprintf(stderr, "pgobin " VERS "  " DATE "
Usage: pgobin -[dhpqUv] [query file]
    -d DataBase name
    -h Hostname of backend server
    -p Port
    -U Username
    -v Debug print
");
    exit(ex);
}

int
main(int argc, char *argv[]) {
    struct connopt *options = malloc(sizeof(struct connopt));
    FILE *in = stdin;
    FILE *out = stdout;
    int c;

    if  (options == NULL)                                  errx(EX_UNAVAILABLE, "Malloc failed #1\n");
    options->dbname = NULL;
    options->host = NULL;
    options->port = NULL;
    options->username = NULL;
    options->query = NULL;
    options->cmd = cmd_null;
    options->intran = 0;
    v_flag = 0;  /*   */
    while ((c = getopt(argc, argv, "c:C:d:h:p:qsSU:v?")) != -1) {
        switch (c) {
        case 'c': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_copyout;
                  options->query = optarg;
                  break;
        case 'C': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_copyin;
                  options->query = optarg;
                  break;
        case 'd': options->dbname = optarg;
                  break;
        case 'h': options->host = optarg;
                  break;
        case 'p': options->port = optarg;
                  break;
        case 'q': v_flag--;
                  break;
        case 's': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_strselect;
                  options->query = optarg;
                  break;
        case 'S': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_binselect;
                  options->query = optarg;
                  break;
        case 'U': options->username = optarg;
                  break;
        case 'v': v_flag++;
                  break;
        case '?': usage(EX_OK);
        default:  usage(EX_USAGE);
    }   }
    argc -= optind;
    argv += optind;
    if  (argc > 0 && argv && *argv) {
        in = fopen(*argv, "r");
        if  (in == NULL)                                           err(EX_NOINPUT, "file %s:", *argv);
    }

    /* Prepare the query */
    switch (options->cmd) {
    case cmd_null:
    case cmd_binselect:
        if  ((options->query = prefile(in, "DECLARE pgobin BINARY CURSOR FOR ", NULL)) == NULL) {
                               if (v_flag >= 0) errx(EX_NOINPUT, "No input\n"); else exit(EX_NOINPUT);
        }
        if  (binselect(options, out)) {
            if  (v_flag >= 0)                                errx(EX_DATAERR, "SELECT BINARY failed");
            exit(EX_DATAERR);
        }
    case cmd_strselect:
        if  ((options->query = prefile(in, NULL, NULL)) == NULL) {
                               if (v_flag >= 0) errx(EX_NOINPUT, "No input\n"); else exit(EX_NOINPUT);
        }
        if  (strselect_p(strselect_q(options), out)) {
            if  (v_flag >= 0)                                       errx(EX_DATAERR, "SELECT failed");
            exit(EX_DATAERR);
    }   }
    PQfinish(options->conn);
    exit(EX_OK);
}
